/* Lista duplamente encadeada */
//laboratorio 12 - estrutuda de dados

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct endereco {
  char nome[30];
  char rua[40];
  char cidade[20];
  char estado[3];
  char cep[10];
  struct endereco *prox;  /* ponteiro para o proximo registro */
  struct endereco *ant;   /* ponteiro para o registro anterior */
} unidade;

struct endereco *inicio;  /* ponteiro para o primeiro registro */
struct endereco *fim;     /* ponteiro para o ultimo registro */
struct endereco *acha(char *);

void Entrada(void), Procura(void), Visualiza(void);
void Apaga(struct endereco **, struct endereco **);
void Armazena(struct endereco *i, struct endereco **inicio,
               struct endereco **fim);
void EntraString(char *, char *, int), Mostra(struct endereco *);
int Menu(void);

void main(void)
  {
  inicio = fim = NULL;  /* inicializa os ponteiros de topo e fundo */

  for(;;)
    {
    switch(Menu())
      {
      case 1: Entrada();
        break;
      case 2: Apaga(&inicio, &fim);
        break;
      case 3: Visualiza();
        break;
      case 4: Procura();              /* encontra um elemento */
        break;
      case 5: exit(0);
      }
    }
  }

/* Seleciona uma operacao. */
Menu(void)
  {
  char s[80];
  int c;

  printf("1. Inserir um nome\n");
  printf("2. Deletar um nome\n");
  printf("3. Visualizar o arquivo\n");
  printf("4. Pesquisar\n");
  printf("5. Sair\n");
  do {
    printf("\nEntre com sua escolha: ");
    gets(s);
    c =atoi(s);
     } while(c<0 || c>5);
  return c;
  }

/* Insere nomes e enderecos */
void Entrada(void)
  {
  struct endereco *info;

  for(;;)
    {
    info = (struct endereco *)malloc(sizeof(unidade)); //para alocar  um espao de memoria
    if(!info) // caso o info nao conseguir alocar memoria imprimir "SEM MEMORIA"
      {                            
      printf("\nsem memoria");
      return;
      }
    EntraString("Entre o nome: ", info->nome, 30);
    if(!info->nome[0]) break;                  /* para a insercao */
    EntraString("Entre a rua: ", info->rua, 40);
    EntraString("Entre a cidade: ", info->cidade, 20);
    EntraString("Entre o estado: ", info->estado, 3);
    EntraString("Entre o cep: ", info->cep, 10);
    Armazena(info, &inicio, &fim);
    }
  }

/* Esta funcao le uma string de comprimento maximo tam e evita que
a string seja ultrapassada. Apresenta uma mensagem de prompt. */
void EntraString(char *prompt, char *s, int tam)
  {
  char p[255];

  do {
    printf(prompt);
    gets(p);
    if(strlen(p)>tam) printf("\nMuito comprido\n");
     } while(strlen(p)>tam);
  strcpy(s, p);
  }

/* Cria uma lista duplamente encadeada ordenada */
void Armazena(
  struct endereco *i,        /* novo elemento */
  struct endereco **inicio,  /* primeiro elemento da lista */
  struct endereco **fim      /* ultimo elemento da lista */
             )
  {
  struct endereco *velho, *p;

  if(*fim==NULL)       /* primeiro elemento da lista */
    {
    i->prox = NULL;
    i->ant = NULL;
    *fim = i;
    *inicio = i;
    return;
    }

  p     = *inicio; /* comeca no topo da lista */
  velho = NULL;

  while(p)
    {
    if(strcmp(p->nome, i->nome)<0) /* verifica a posicao do novo elemento */
      {                            /* para que o mesmo fique em ordem     */
      velho = p;                   /* alfabetica                          */
      p = p->prox;
      }
    else                           /* caso o nome seja maior que o elemento */
      {                            /* apontado por p                        */
      if(p->ant)
        {
        p->ant->prox = i;
        i->prox = p;
        i->ant = p->ant;
        p->ant = i;
        return;
        }
      i->prox = p;   /* caso a nova informacao seja o novo primeiro elemento */
      i->ant = NULL;
      p->ant = i;
      *inicio = i;
      return;
      }
    }
  velho->prox = i;  /* coloca no final */
  i->prox = NULL;
  i->ant = velho;
  *fim = i;
  }

/* Remove um elemento da lista */
void Apaga(struct endereco **inicio, struct endereco **fim)
  {
  struct endereco *info, *acha();
  char s[80];

  EntraString("Entre o nome: ", s, 30);
  info = acha(s);
  if(info)
    {
    if(*inicio==info)         /* se a informao estiver no inicio */
      {
      *inicio=info->prox;
      if(*inicio)
        (*inicio)->ant = NULL;
      else
        *fim = NULL;
      }
    else                      /* se a informacao estiver no meio ou no fim */
      {
      info->ant->prox = info->prox;
      if(info!=*fim)
        info->prox->ant = info->ant;
      else
        *fim = info->ant;
      }
    free(info);              /* devolve memoria para o sistema */
    }
  }

/* Encontra um endereco */
struct endereco *acha(char *nome)
  {
  struct endereco *info;

  info = inicio;
  while(info)
    {
    if(!strcmp(nome, info->nome)) return info;
    info = info->prox;  /* obtem novo endereco */
    }
  printf("Nome nao encontrado.\n");
  return NULL; /* nao encontrou */
  }

/* Mostra a lista completa */
void Visualiza(void)
  {
  struct endereco *info;
  info = inicio;
  while(info)
    {
    Mostra(info);
    info = info->prox;   /* obtem proximo endereco */
    }
  printf("\n\n");
  }

/* Esta funcao imprime os campos de cada endereco. */
void Mostra(struct endereco *info)
  {
  printf("Nome:\n%s\n\n", info->nome);
  printf("Rua:\n %s\n\n", info->rua);
  printf("Cidade:\n%s\n\n", info->cidade);
  printf("Estado:\n%s\n\n", info->estado);
  printf("Cep:\n%s\n\n", info->cep);
  printf("\n\n");
  }

/* Procura por um nome na lista. */
void Procura(void)
  {
  char nome[40];
  struct endereco *info, *acha();

  printf("Entre com o nome a Procurar: ");
  gets(nome);
  info = acha(nome);
  if(!info) printf("nao encontrou\n");
  else Mostra(info);
  }

